"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.$removedefault = exports.$setdefault = exports.$default = exports.$language = exports.$country = exports.$locales = exports.$tmp = exports.$localpath = exports.TSDefaults = void 0;
const commons_1 = require("./commons");
const types_1 = require("./types");
const fs_1 = require("./fs");
const os_1 = __importDefault(require("os"));
class TSDefaults {
    constructor() {
        this.tmpDirectory = os_1.default.tmpdir();
        this.defaultLanguage = types_1.Languages.fr;
        this._values = {};
        this._localizations = {};
        this.defaultPath = __dirname;
        for (let sf in TSDefaults.__subfolders) {
            if ((0, fs_1.$filename)(this.defaultPath) === sf) {
                this.defaultPath = (0, fs_1.$dir)(this.defaultPath);
            }
        }
        this._countriesMap = new Map();
        Object.keys(types_1.Countries).forEach(e => this._countriesMap.set(e, e));
        this._languagesMap = new Map();
        Object.keys(types_1.Languages).forEach(e => this._languagesMap.set(e, e));
    }
    country(s) {
        const v = (0, commons_1.$trim)(s);
        if (v.length !== 2) {
            return null;
        }
        const ret = this._countriesMap.get(v.toUpperCase());
        return (0, commons_1.$ok)(ret) ? ret : null;
    }
    language(s) {
        if (!(0, commons_1.$ok)(s)) {
            return this.defaultLanguage;
        }
        const v = (0, commons_1.$trim)(s);
        if (v.length !== 2) {
            return null;
        }
        const ret = this._languagesMap.get(v.toLowerCase());
        return (0, commons_1.$ok)(ret) ? ret : null;
    }
    addLocalizations(lang, loc) {
        if ((0, commons_1.$isobject)(loc)) {
            let actualLocalization = this._localizations[lang];
            if (!(0, commons_1.$ok)(actualLocalization)) {
                this._localizations[lang] = actualLocalization;
            }
            else {
                this._localizations[lang] = Object.assign(Object.assign({}, actualLocalization), loc);
            }
        }
    }
    localizations(lang) {
        let ret = this._localizations[lang || this.defaultLanguage];
        return ret || {};
    }
    locales(lang) {
        if (!(0, commons_1.$ok)(lang) || !(0, commons_1.$ok)(TSDefaults.__locales[lang])) {
            lang = this.defaultLanguage;
        }
        return TSDefaults.__locales[lang];
    }
    static setSubfolders(folders) {
        this.__subfolders = folders;
    }
    setDefaultLanguage(l) {
        if ((0, commons_1.$ok)(TSDefaults.__locales[l])) {
            this.defaultLanguage = l;
        }
        return this.defaultLanguage;
    }
    setTmpDirectory(path) {
        if ((0, fs_1.$isdirectory)(path)) {
            this.tmpDirectory = path;
        }
    }
    // these 3 methods permits using software to store global values on unique Defaults instance
    setValue(key, value) {
        if ((0, commons_1.$length)(key)) {
            if ((0, commons_1.$ok)(value)) {
                this._values[key] = value;
            }
            else if ((0, commons_1.$ok)(this._values[key])) {
                delete this._values[key];
            }
        }
    }
    getValue(key) { return this._values[key]; }
    static defaults() {
        if (!this.__instance) {
            this.__instance = new TSDefaults();
        }
        return this.__instance;
    }
}
exports.TSDefaults = TSDefaults;
TSDefaults.__subfolders = ['utils', 'tests', 'dist'];
/**
 * LOCALES (mostly date/time locales) are set in 6 common world's languages
 * - english (en)
 * - french (fr),
 * - spanish (es),
 * - german (de),
 * - italian (it)
 * - portuguese (pt)
 */
TSDefaults.__locales = {
    en: {
        months: ["January", "February", "March", "April", "May", "June", "July", "August", "Septembre", "Octobre", "Novembre", "Decembre"],
        shortMonths: ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"],
        days: ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"],
        shortDays: ["Sun.", "Mon.", "Tue.", "Wed.", "Thu.", "Fri.", "Sat."],
        startingWeekDay: 0,
        dateTimeFormat: "%m/%d/%Y %H:%M:%S",
        dateFormat: "%m/%d/%Y",
        shortDateFormat: "%m/%d/%y",
        shortDateTimeFormat: "%m/%d/%y %H:%M:%S",
        timeFormat: "%H:%M:%S",
        partialTimeFormat: "%H:%M",
        language: { fr: "anglais", en: "english", de: "englisch", it: "inglese", es: "inglés", pt: "inglês" },
        ampm: ['AM', 'PM']
    },
    fr: {
        months: ["janvier", "février", "mars", "avril", "mai", "juin", "juillet", "août", "septembre", "octobre", "novembre", "décembre"],
        shortMonths: ["jan.", "fév.", "mars", "avr.", "mai", "juin", "juil.", "août", "sept.", "oct.", "nov.", "déc."],
        days: ["dimanche", "lundi", "mardi", "mercredi", "jeudi", "vendredi", "samedi"],
        shortDays: ["Di", "Lu", "Ma", "Me", "Je", "Ve", "Sa"],
        startingWeekDay: 1,
        dateTimeFormat: "%d/%m/%Y %H:%M:%S",
        dateFormat: "%d/%m/%Y",
        shortDateFormat: "%d/%m/%y",
        shortDateTimeFormat: "%d/%m/%y %H:%M:%S",
        timeFormat: "%H:%M:%S",
        partialTimeFormat: "%H:%M",
        language: { fr: "français", en: "french", de: "französisch", it: "francese", es: "francés", pt: "francês" },
        ampm: ['AM', 'PM']
    },
    es: {
        shortDays: ["Dom", "Lun", "Mar", "Mie", "Jue", "Vie", "Sab"],
        days: ["domingo", "lunes", "martes", "miércoles", "jueves", "viernes", "sábado"],
        shortMonths: ["ene", "feb", "mar", "abr", "may", "jun", "jul", "ago", "sep", "oct", "nov", "dic"],
        months: ["enero", "febrero", "marzo", "abril", "mayo", "junio", "julio", "agosto", "septiembre", "octubre", "noviembre", "diciembre"],
        shortDateFormat: "%d/%m/%y",
        dateFormat: "%d/%m/%Y",
        timeFormat: "%H:%M:%S",
        partialTimeFormat: "%H:%M",
        dateTimeFormat: "%d/%m/%Y %H:%M:%S",
        shortDateTimeFormat: "%d/%m/%y %H:%M:%S",
        startingWeekDay: 1,
        language: { fr: "espagnol", en: "spanish", de: "spanisch", it: "spagnolo", es: "español", pt: "espanhol" },
        ampm: ['AM', 'PM']
    },
    de: {
        months: ["Januar", "Februar", "März", "April", "Mai", "Juni", "Juli", "August", "September", "Oktober", "November", "Dezember"],
        shortMonths: ["Jan", "Feb", "Mrz", "Apr", "Mai", "Jun", "Jul", "Aug", "Sep", "Okt", "Nov", "Dez"],
        days: ["Sonntag", "Montag", "Dienstag", "Mittwoch", "Donnerstag", "Freitag", "Samstag"],
        shortDays: ["So", "Mo", "Di", "Mi", "Do", "Fr", "Sa"],
        startingWeekDay: 1,
        dateTimeFormat: "%%e.%m.%Y %H:%M:%S",
        dateFormat: "%e.%m.%Y",
        shortDateFormat: "%e.%m.%y",
        shortDateTimeFormat: "%%e.%m.%y %H:%M:%S",
        timeFormat: "%H:%M:%S",
        partialTimeFormat: "%H:%M",
        language: { fr: "allemand", en: "german", de: "deutsch", it: "tedesco", es: "alemán", pt: "alemão" },
        ampm: ['AM', 'PM']
    },
    it: {
        shortDays: ["dom.", "lun.", "mar.", "mer.", "gio.", "ven.", "sab."],
        days: ["domenica", "lunedi", "martedì", "mercoledì", "giovedi", "venerdì", "sabato"],
        shortMonths: ["gen", "feb", "mar", "apr", "mag", "giu", "lug", "ago", "set", "ott", "nov", "dic"],
        months: ["gennaio", "febbraio", "marzo", "aprile", "maggio", "giugno", "luglio", "agosto", "settembre", "ottobre", "novembre", "dicembre"],
        shortDateFormat: "%d/%m/%y",
        dateFormat: "%d/%m/%Y",
        timeFormat: "%H:%M:%S",
        partialTimeFormat: "%H:%M",
        dateTimeFormat: "%d/%m/%Y %H:%M:%S",
        shortDateTimeFormat: "%d/%m/%y %H:%M:%S",
        startingWeekDay: 1,
        language: { fr: "italien", en: "italian", de: "italienisch", it: "italiano", es: "italiano", pt: "italiano" },
        ampm: ['AM', 'PM']
    },
    pt: {
        shortDays: ["Dom", "Seg", "Ter", "Qua", "Qui", "Sex", "Sab"],
        days: ["Domingo", "Segunda-feira", "Terça-feira", "Quarta-feira", "Quinta-feira", "Sexta-feira", "Sábado"],
        shortMonths: ["Jan", "Fev", "Mar", "Abr", "Mai", "Jun", "Jul", "Agos", "Set", "Out", "Nov", "Dez"],
        months: ["Janeiro", "Fevereiro", "Março", "Abril", "Maio", "Junho", "Julho", "Agosto", "Setembro", "Outubro", "Novembro", "Dezembro"],
        shortDateFormat: "%d-%m-%y",
        dateFormat: "%d-%m-%Y",
        timeFormat: "%H:%M:%S",
        partialTimeFormat: "%H:%M",
        dateTimeFormat: "%d-%m-%Y %H:%M:%S",
        shortDateTimeFormat: "%d-%m-%y %H:%M:%S",
        startingWeekDay: 0,
        language: { fr: "portuguais", en: "portuguese", de: "portugiesisch", it: "portoghese", es: "portugués", pt: "portugueses" },
        ampm: ['AM', 'PM']
    },
};
function $localpath() { return TSDefaults.defaults().defaultPath; }
exports.$localpath = $localpath;
function $tmp() { return TSDefaults.defaults().tmpDirectory; }
exports.$tmp = $tmp;
function $locales(lang) { return TSDefaults.defaults().locales(lang); }
exports.$locales = $locales;
function $country(s) { return TSDefaults.defaults().country(s); }
exports.$country = $country;
function $language(s) { return TSDefaults.defaults().language(s); }
exports.$language = $language;
// to get default language, tou call $language() with no parameters or TSDefaults.defaults().defaultLanguage 
// function to manage your own global defaults.
// warning: all of this defaults are stored in memory
function $default(key) { return TSDefaults.defaults().getValue(key); }
exports.$default = $default;
function $setdefault(key, value = undefined) { return TSDefaults.defaults().setValue(key, value); }
exports.$setdefault = $setdefault;
function $removedefault(key) { return TSDefaults.defaults().setValue(key, undefined); }
exports.$removedefault = $removedefault;
//# sourceMappingURL=tsdefaults.js.map